# frozen_string_literal: true

module PlanetExpress
  class DiscountBudget
    attr_reader :budget

    def initialize(budget:)
      @budget = budget
      @applied_discounts = []
    end

    def register_applied_discount(discount_amount)
      possible_discount = maximum_possible_discount(discount_amount)
      applied_discounts.push(possible_discount)
      possible_discount
    end

    def maximum_possible_discount(discount_amount)
      [available_budget, discount_amount].min
    end

    def available_budget
      budget - spent_budget
    end

    def remaining_budget?
      !available_budget.zero?
    end

    private

    attr_reader :applied_discounts

    def spent_budget
      applied_discounts.sum
    end
  end
end
