module PlanetExpress
  class SShipmentDiscountRule < Rule
    protected

    def applicable_to?(shipment)
      s_shipment?(shipment) && more_expensive_than_cheapest_shipping_option?(shipment)
    end

    def discount_for(shipment)
      Discount.new(discount_amount(shipment), self)
    end

    private

    def cheapest_available_shipping_price(shipment)
      shipping_option_repository.lowest_price_for_package_size(package_size(shipment))
    end

    def more_expensive_than_cheapest_shipping_option?(shipment)
      price(shipment) > cheapest_available_shipping_price(shipment)
    end

    def package_size(shipment)
      shipment.shipping_option.package_size
    end

    def s_shipment?(shipment)
      shipment.shipping_option.s_shipment?
    end

    def discount_amount(shipment)
      desired_amount = price(shipment) - cheapest_available_shipping_price(shipment)
      discount_budget(shipment).maximum_possible_discount(desired_amount)
    end
  end
end
