# frozen_string_literal: true

RSpec.describe PlanetExpress::LShipmentDiscountRule do
  let(:budget_repo) { PlanetExpress::DiscountBudgetForMonthRepository.new }
  let(:shipping_option_repo) { PlanetExpress::ShippingOptionRepository.new }
  let(:shipment_repo) { PlanetExpress::ShipmentRepository.new(data: []) }

  subject do
    described_class.new(
      discount_budget_for_month_repository: budget_repo,
      shipping_option_repository: shipping_option_repo,
      shipment_repository: shipment_repo)
  end

  let(:correct_shipping_option) do
    shipping_option_repo.find_by_provider_and_package_size('LP', 'L')
  end

  let(:applicable_shipment) do
    PlanetExpress::Shipment.new(Date.today, correct_shipping_option)
  end

  context 'l_shipment' do
    context 'not the third shipment' do
      it 'does not apply the discount' do
        expect do
          subject.apply_to(applicable_shipment)
        end.to_not(change { applicable_shipment.discounts })
      end
    end

    context 'third shipment for LP' do
      context 'LP provider' do
        before(:each) do
          shipment_repo.add(applicable_shipment)
          shipment_repo.add(applicable_shipment)
        end

        it 'does apply the discount' do
          expect do
            subject.apply_to(applicable_shipment)
          end.to(change { applicable_shipment.discounts.length }.by(1))

          discount = applicable_shipment.discounts.last
          expect(discount.amount).to eq(applicable_shipment.shipping_option.price_in_cents)
        end

        it 'updates the discount budget' do
          expected_difference = -applicable_shipment.shipping_option.price_in_cents

          expect do
            subject.apply_to(applicable_shipment)
          end.to(change { budget_repo.for_month(applicable_shipment.month).available_budget }.by(expected_difference))
        end
      end

      context 'non LP provider' do
        before(:each) do
          shipment_repo.add(applicable_shipment)
          shipment_repo.add(applicable_shipment)
        end
        let(:shipping_option) do
          shipping_option_repo.find_by_provider_and_package_size('MR', 'L')
        end
        let(:shipment) do
          PlanetExpress::Shipment.new(Date.today, shipping_option)
        end

        it 'does not update the discounts' do
          expect do
            subject.apply_to(shipment)
          end.to_not(change { shipment.discounts })
        end
      end
    end
  end
end
