# frozen_string_literal: true

RSpec.describe PlanetExpress::ShipmentFormatter do
  subject do
    described_class.new
  end

  describe '#format' do
    context 'invalid shipment' do
      let(:shipment) do
        PlanetExpress::InvalidShipment.new("Invalid Shipment")
      end

      it 'returns a well formatted string' do
        expect(subject.format(shipment)).to eq('Invalid Shipment IGNORED')
      end
    end

    context 'valid shipment' do
      let(:date) { Date.today }
      let(:shipping_option) do
        build(:shipping_option,
              provider: 'T',
              package_size: 'S',
              price_in_cents: 120)
      end

      let(:shipment) do
        PlanetExpress::Shipment.new(date, shipping_option)
      end

      it 'returns a well formatted string' do
        result = subject.format(shipment)
        parts = result.split(' ')

        expect(parts[0]).to eq(date.iso8601)
        expect(parts[1]).to eq(shipping_option.package_size)
        expect(parts[2]).to eq(shipping_option.provider)
        expect(parts[3]).to eq('1.20')
        expect(parts[4]).to eq('-')
      end

      context 'with discounts' do
        before(:each) do
          shipment.add_discount(PlanetExpress::Discount.new(50, nil))
          shipment.add_discount(PlanetExpress::Discount.new(25, nil))
        end

        it 'includes the discounts' do
          result = subject.format(shipment)
          parts = result.split(' ')

          expect(parts[0]).to eq(date.iso8601)
          expect(parts[1]).to eq(shipping_option.package_size)
          expect(parts[2]).to eq(shipping_option.provider)
          expect(parts[3]).to eq('0.45')
          expect(parts[4]).to eq('0.75')
        end
      end
    end
  end
end
